;; -*- mode: common-lisp -*-
;;
;; Copyright (C) 2006 Franz Inc, Oakland, CA.  All rights reserved.
;;
;; This code is free software; you can redistribute it and/or
;; modify it under the terms of the version 2.1 of
;; the GNU Lesser General Public License as published by 
;; the Free Software Foundation, as clarified by the Franz
;; preamble to the LGPL found in
;; http://opensource.franz.com/preamble.html.
;;
;; This code is distributed in the hope that it will be useful,
;; but without any warranty; without even the implied warranty of
;; merchantability or fitness for a particular purpose.  See the GNU
;; Lesser General Public License for more details.
;;
;; Version 2.1 of the GNU Lesser General Public License can be
;; found at http://opensource.franz.com/license.html.
;; If it is not present, you can access it from
;; http://www.gnu.org/copyleft/lesser.txt (until superseded by a newer
;; version) or write to the Free Software Foundation, Inc., 59 Temple
;; Place, Suite 330, Boston, MA  02111-1307  USA
;;
;; $Id$

(in-package :user)

(defun start-telnet-server (port)
  (loop
    (let ((socket (socket:make-socket :connect :passive :local-port port
				      :reuse-address t)))
      (unwind-protect
	  (loop
	    (let ((connection (socket:accept-connection socket))
		  from)
	      (handler-case
		  (progn
		    (setq from (or (socket:ipaddr-to-hostname
				    (socket:remote-host connection))
				   (socket:ipaddr-to-dotted
				    (socket:remote-host connection))))
		    (logit "telnet server: new connection from ~a"
			   from)
		    (format connection "
WARNING: do not use :exit or (exit).  You will kill the web
         server if you do.  Use ~s to quit."
			    '(quit))
		    (force-output connection) 
		    (mp:process-run-function
		     "telnet session"
		     'start-telnet-session connection from))
		(error ()
		  (ignore-errors (close connection))))))
	(ignore-errors (close socket))))))

(defvar *in-telnet-session* nil)

(defun start-telnet-session (s from)
  (unwind-protect
      (catch 'end-telnet-session
	(let ((*in-telnet-session* t))
	  (tpl:start-interactive-top-level
	   s 'tpl:top-level-read-eval-print-loop nil)))
    (ignore-errors (close s)))
  (logit "telnet server: closing connection from ~a" from))

(defun quit ()
  (throw 'end-telnet-session nil))

(defvar *exit-wrapped* nil)

(when (not *exit-wrapped*)
  (flet ((msg () (format t "Use ~s instead of exit.~%" '(quit))))
    (def-fwrapper exit-wrapper (&optional status &rest args)
      (declare (ignore args))
      (if* *in-telnet-session*
	 then (msg)
	 else (call-next-fwrapper)))
  
    (fwrap 'excl:exit :lfs 'exit-wrapper)
    (fwrap 'tpl::exit-command :lfs 'exit-wrapper))) 

