;; -*- mode: common-lisp -*-
;;
;; Copyright (C) 2006 Franz Inc, Oakland, CA.  All rights reserved.
;;
;; This code is free software; you can redistribute it and/or
;; modify it under the terms of the version 2.1 of
;; the GNU Lesser General Public License as published by 
;; the Free Software Foundation, as clarified by the Franz
;; preamble to the LGPL found in
;; http://opensource.franz.com/preamble.html.
;;
;; This code is distributed in the hope that it will be useful,
;; but without any warranty; without even the implied warranty of
;; merchantability or fitness for a particular purpose.  See the GNU
;; Lesser General Public License for more details.
;;
;; Version 2.1 of the GNU Lesser General Public License can be
;; found at http://opensource.franz.com/license.html.
;; If it is not present, you can access it from
;; http://www.gnu.org/copyleft/lesser.txt (until superseded by a newer
;; version) or write to the Free Software Foundation, Inc., 59 Temple
;; Place, Suite 330, Boston, MA  02111-1307  USA
;;
;; $Id$

(in-package :user)

(defparameter *reload-hook* nil)

(defun reload ()
  (let ((config-file (merge-pathnames "config.cl" *sample-server-root*)))
    (when (probe-file config-file) (load config-file)))
  
  (unpublish :all t :server *server*)
  (without-interrupts
    ;;; So streams don't get lost when logfile locations are reconfigured.
    (load (or (probe-file (if (featurep :www-release)
			      (merge-pathnames "publish.cl" *sample-server-root*)
			    (merge-pathnames (make-pathname :directory '(:relative "src")
							    :name "publish"
							    :type "cl"))))
	      (error "can't find publish.cl")))
    (when *reload-hook*
      (dolist (fn *reload-hook*)
	(funcall fn)))
    
    (init-log-flush-thread)))

(defun error-die (format &rest args)
  (if* *debug*
     then (apply #'break format args)
     else (apply #'format t format args)
	  (fresh-line)
	  (exit -1 :no-unwind t :quiet t)))

(defun logit (format-string &rest args)
  (let ((s *initial-terminal-io*)
	(*print-length* 20)
	(*print-level* 10))
    (multiple-value-bind (csec cmin chour cday cmonth cyear)
	(decode-universal-time (get-universal-time))
      (format
       s
       "~&~a: ~2,'0d/~2,'0d/~2,'0d - ~2,'0d:~2,'0d:~2,'0d - "
       (mp:process-name sys:*current-process*)
       cmonth cday (mod cyear 100)
       chour cmin csec)
      (apply #'format s format-string args)
      (terpri s)
      (force-output s))))
